# -*- coding: utf-8 -*-
#
# Copyright (c) 2021, HILSTER GmbH, Germany - https://hilster.io
# All rights reserved.
#
from htf.daqmx.core import DAQError
from htf.daqmx.task import Task
from ctypes import *
from htf.daqmx.api import int32,\
    float64, DAQmx_Val_GroupByChannel, DAQmx_Val_FiniteSamps, DAQmx_Val_Rising,\
    DAQmx_Val_Volts, DAQmx_Val_Cfg_Default, uInt64, DAQmx_Val_ContSamps,\
    DAQmx_Val_OnBrdMemMoreThanHalfFull
from htf.mixins.delegation import DelegateMixin


class AnalogVoltageInputPort(DelegateMixin):
    """
    ``AnalogVoltageInputPort`` is used to access a single analog input port
    on a DAQmx device.

    A DAQmx task is automatically created and deleted in the constructor and
    destructor.
    """

    def __init__(self, descriptor,
                 minimum_voltage=-10.0,
                 maximum_voltage=10.0,
                 number_of_lines=32,
                 number_of_samples=2,
                 sample_frequency=1000.0,
                 timeout=1.0,
                 terminal_config=DAQmx_Val_Cfg_Default,
                 continuous_mode=False):
        """
        Args:
            descriptor (str): The DAQmx device and line descriptor, eg.
                ``cDaq1Mod1/line0``.
            minimum_voltage=-10.0 (float): the minimum voltage.
            maximum_voltage=10.0 (float): the maximum voltage.
            number_of_lines=32 (int): the number of lines defined in the
                descriptor. For example the descriptor "cDaq1Mod3/ai0:31"
                defines 32 lines.
            number_of_samples=2 (int): the number of samples to acquire for one
                average value.
            sample_frequency=1000.0 (float): the sample frequency in Hertz.
            timeout=1.0 (float): the timeout when acquiring a new value in
                seconds.
            terminal_config=DAQmx_Val_Cfg_Default: the terminal configuration
                for DAQmxCreateAIVoltageChan(). Possible values: DAQmx_Val_RSE,
                DAQmx_Val_NRSE, DAQmx_Val_Diff, DAQmx_Val_PseudoDiff.
            continuous_mode=False: if set to ``True`` continuous sampling is
                done and get() becomes blocking. Simply read samples in a loop.
        """
        task_name = "AI_" + descriptor.replace("/", "_")
        self.number_of_samples = number_of_samples
        self.sample_frequency = sample_frequency
        self.number_of_lines = number_of_lines
        self.continuous_mode = continuous_mode
        self.timeout = timeout
        self.data = cast(
            (float64 * self.number_of_samples * self.number_of_lines)(),
            POINTER(float64))
        self.read = int32(0)
        self.task = Task(task_name)

        if not isinstance(descriptor, bytes):
            descriptor = descriptor.encode()

        self.task.CreateAIVoltageChan(descriptor,
                                      None,
                                      terminal_config,
                                      float64(minimum_voltage),
                                      float64(maximum_voltage),
                                      DAQmx_Val_Volts,
                                      None)

        if self.continuous_mode is False:
            self.task.CfgSampClkTiming(
                None,
                float(self.sample_frequency),
                DAQmx_Val_Rising,
                DAQmx_Val_FiniteSamps,
                uInt64(self.number_of_samples * self.number_of_lines))
        else:
            self.task.CfgSampClkTiming(
                None,
                float(self.sample_frequency),
                DAQmx_Val_Rising, DAQmx_Val_ContSamps,
                uInt64(self.number_of_samples * self.number_of_lines))
            self.task.SetAIDataXferReqCond(descriptor,
                                           DAQmx_Val_OnBrdMemMoreThanHalfFull)
            self.task.SetBufInputBufSize(2 * int(self.number_of_samples))

    def get(self):
        """
        Return the current analog value.

        Returned data layout::

            [
                [line0 sample0, line0 sample1, ..., line0 sampleN-1],
                [line1 sample0, line1 sample1, ..., line1 sampleN-1],
                ...
                [lineM-1 sample0, lineM-1 sample1, ..., lineM-1 sampleN-1]
            ]
        """
        self.task.ReadAnalogF64(self.number_of_samples,
                                float64(self.timeout),
                                DAQmx_Val_GroupByChannel,
                                self.data,
                                self.number_of_samples * self.number_of_lines,
                                byref(self.read),
                                None)

        data = []

        for line in range(self.number_of_lines):
            lineData = []
            for sample in range(self.number_of_samples):
                lineData.append(self.data[line * self.number_of_samples + sample])
            data.append(lineData)
        return data

    def set(self, state=True):
        raise NotImplementedError(
            "There is no meaningful implementation for "
            "a 'set' method in the context of this class.")

    def close(self):
        """
        Close the abstractor by clearing the task.
        """
        try:
            self.task.ClearTask()
        except DAQError:
            pass

    def __del__(self):
        self.close()

    def __str__(self):
        return str(self.get())


if __name__ == "__main__":
    ai0 = AnalogVoltageInputPort("Dev1/ai0:0", number_of_samples=10, sample_frequency=10, continuous_mode=True)

    for ii in range(100000):
        samples = ai0.get()[0]
        for sample in samples:
            print(sample)
        # time.sleep(.5)#
