# -*- coding: utf-8 -*-
#
# Copyright (c) 2021, HILSTER GmbH, Germany - https://hilster.io
# All rights reserved.
#
from htf.daqmx.core import DAQError
from htf.daqmx.task import Task
from ctypes import *
from htf.daqmx.api import float64, DAQmx_Val_Volts, DAQmx_Val_Cfg_Default, DAQmx_Val_Rising,\
    DAQmx_Val_FiniteSamps, DAQmx_Val_GroupByChannel, int32
from htf.mixins.delegation import DelegateMixin


class AnalogVoltageInputSingleLineAccurate(DelegateMixin):
    """
    ``AnalogVoltageInputSingleLineAccurate`` is used to access a single analog input line
    on a DAQmx device. It acquires some samples and builds the average value of them.

    A DAQmx task is automatically created and deleted in the constructor and destructor.
    """

    def __init__(self, descriptor,
                 minimum_voltage=-10.0,
                 maximum_voltage=10.0,
                 number_of_samples=2,
                 sample_frequency=1000.0,
                 timeout=1.0,
                 terminal_config=DAQmx_Val_Cfg_Default):
        """
        Args:
            descriptor (str): The DAQmx device and line descriptor, eg. `cDaq1Mod1/line0`
            minimum_voltage=-10.0 (float): the minimum voltage.
            maximum_voltage=10.0 (float): the maximum voltage.
            number_of_samples=2 (int): the number of samples to acquire for one average value.
            sample_frequency=1000.0 (float): the sample frequency in Hertz.
            timeout=1.0 (float): the timeout when acquiring a new value in seconds.
            terminal_config=DAQmx_Val_Cfg_Default: the terminal configuration for DAQmxCreateAIVoltageChan().
                Possible values: DAQmx_Val_RSE, DAQmx_Val_NRSE, DAQmx_Val_Diff, DAQmx_Val_PseudoDiff.
        """
        task_name = "AI_" + descriptor.replace("/", "_")
        self.number_of_samples = number_of_samples
        self.sample_frequency = sample_frequency
        self.timeout = timeout
        # self.data = cast((float64 * self.number_of_samples)(), POINTER(float64))
        self.data = cast((float64 * self.number_of_samples)(), POINTER(float64))
        self.value = 0.0
        self.read = int32(0)
        self.task = Task(task_name)

        if not isinstance(descriptor, bytes):
            descriptor = descriptor.encode()

        self.task.CreateAIVoltageChan(descriptor,
                                      None,
                                      terminal_config,
                                      float64(minimum_voltage),
                                      float64(maximum_voltage),
                                      DAQmx_Val_Volts,
                                      None)
        self.task.CfgSampClkTiming(None,
                                   float(self.sample_frequency),
                                   DAQmx_Val_Rising,
                                   DAQmx_Val_FiniteSamps,
                                   self.number_of_samples)

    def get(self):
        """
        Return the current pin value.
        """
        self.task.ReadAnalogF64(self.number_of_samples,
                                timeout=float64(self.timeout),
                                fillMode=DAQmx_Val_GroupByChannel,
                                readArray=self.data,
                                arraySizeInSamps=self.number_of_samples,
                                sampsPerChanRead=byref(self.read),
                                reserved=None)
        assert self.read.value == self.number_of_samples

        # build the average value
        s = 0.0
        for i in range(self.number_of_samples):
            s += self.data[i]

        self.value = s / float(self.number_of_samples)

        return self.value

    def set(self, value=True):
        raise NotImplementedError(
            "There is no meaningful implementation for a 'set' method in the context of this class.")

    def close(self):
        """
        Close the abstractor by clearing the task.
        """
        try:
            self.task.ClearTask()
        except DAQError:
            pass

    def __del__(self):
        self.close()

    def __str__(self):
        return str(self.get())


if __name__ == "__main__":
    import time
    ai0 = AnalogVoltageInputSingleLineAccurate(
        "Dev1/ai0", minimum_voltage=0, maximum_voltage=5.0, number_of_samples=100, timeout=12.0, sample_frequency=100)

    for ii in range(1000):
        print("ai0:", ai0.get())
        time.sleep(1.0)
