# -*- coding: utf-8 -*-
#
# Copyright (c) 2021, HILSTER GmbH, Germany - https://hilster.io
# All rights reserved.
#
from htf.daqmx.core import DAQError
from htf.daqmx.task import Task
from ctypes import *
from htf.daqmx.api import float64, DAQmx_Val_Rising, DAQmx_Val_Falling,\
    DAQmx_Val_Hz, DAQmx_Val_LowFreq1Ctr
from htf.mixins.delegation import DelegateMixin


class FrequencyInput(DelegateMixin):
    """
    ``FrequencyInput`` is used to measure a low frequency in hertz on a DAQmx
    device.

    A DAQmx task is automatically created and deleted in the constructor and
    destructor.
    """

    def __init__(self, descriptor, f_min, f_max, measurement_time, rising_edge=True):
        """
        Args:
            descriptor (str): The DAQmx device and line descriptor, eg.
                `cDaq1Mod1/ctr0`
            f_min (float): The minimum frequency in the input signal in Hertz.
            f_min (float): The maximum frequency in the input signal in Hertz.
            rising_edge=True (bool): If set to True rising edges are counted
                else falling edges are counted.
            measurement_time (float): The measurement time in seconds.
        """
        task_name = "FREQLOW_" + descriptor.replace("/", "_")
        self.f_min = f_min
        self.f_max = f_max
        self.measurement_time = measurement_time
        self.data = float64(0.0)
        self.task = Task(task_name)

        self.edge = DAQmx_Val_Rising
        if not rising_edge:
            self.edge = DAQmx_Val_Falling

        if not isinstance(descriptor, bytes):
            descriptor = descriptor.encode()

        self.task.CreateCIFreqChan(descriptor,
                                   None,
                                   minVal=self.f_min,
                                   maxVal=self.f_max,
                                   units=DAQmx_Val_Hz,
                                   edge=self.edge,
                                   measMethod=DAQmx_Val_LowFreq1Ctr,
                                   measTime=float64(self.measurement_time),
                                   divisor=4,
                                   customScaleName=None)
        self.task.start()

    def get(self):
        """
        Return the current frequency.
        """
        self.task.ReadCounterScalarF64(float64(1.0), byref(self.data), None)

        return self.data.value

    def close(self):
        """
        Close the abstractor by clearing the task.
        """
        try:
            self.task.ClearTask()
        except DAQError:
            pass

    def __del__(self):
        self.close()

    def __str__(self):
        return str(self.get())


if __name__ == "__main__":
    import time

    f0 = FrequencyInput(descriptor="Dev1/ctr0", f_min=1000.0, f_max=10000.0, measurement_time=0.001, rising_edge=True)

    for ii in range(100):
        print(f0.get())
        time.sleep(0.1)
